// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/driver/io.c $
// $Rev: 46648 $
// $Date: 2019-12-18 13:08:31 -0600 (Wed, 18 Dec 2019) $

// HPDI32: Device Driver: source file

#include "main.h"



/******************************************************************************
*
*	Function:	io_auto_start
*
*	Purpose:
*
*		Perform any necessary Auto-Start activities.
*		We are only called when Auto-Start is enabled.
*
*	Arguments:
*
*		dev		The data for the device of interest.
*
*		io		The I/O data structure.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void io_auto_start(dev_data_t* dev, dev_io_t* io)
{
	#define	RX_ENABLE	HPDI32_BCR_RX_ENABLE
	#define	TX_ENABLE	HPDI32_BCR_TX_ENABLE
	#define	TX_START	HPDI32_BCR_TX_START

	if (io->is_tx)
	{
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, TX_ENABLE, TX_ENABLE);
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, TX_START, TX_START);
	}
	else
	{
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, RX_ENABLE, RX_ENABLE);
	}
}



/******************************************************************************
*
*	Function:	dev_io_close
*
*	Purpose:
*
*		Cleanup the I/O stuff for the device as it is being closed.
*
*	Arguments:
*
*		dev	The data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_io_close(dev_data_t* dev)
{
	int	i;

	for (i = 0; i < DEV_IO_STREAM_QTY; i++)
	{
		if (dev->io.io_streams[i])
		{
			if (dev->io.io_streams[i]->dev_io_close)
			{
				(dev->io.io_streams[i]->dev_io_close)(dev, dev->io.io_streams[i]);
			}
		}
	}
}



/******************************************************************************
*
*	Function:	dev_io_create
*
*	Purpose:
*
*		Perform I/O based initialization as the driver is being loaded.
*
*	Arguments:
*
*		dev		The data for the device of interest.
*
*	Returned:
*
*		0		All went well.
*		< 0		The error status for the problem encounterred.
*
******************************************************************************/

int dev_io_create(dev_data_t* dev)
{
	int	ret		= 0;
	int	tmp;

	dev->io.io_streams[0]	= &dev->io.rx;
	dev->io.io_streams[1]	= &dev->io.tx;

	tmp	= dev_read_create(dev, &dev->io.rx);
	ret	= ret ? ret : tmp;

	tmp	= dev_write_create(dev, &dev->io.tx);
	ret	= ret ? ret : tmp;

	return(ret);
}



/******************************************************************************
*
*	Function:	dev_io_destroy
*
*	Purpose:
*
*		Perform I/O based cleanup as the driver is being unloaded.
*
*	Arguments:
*
*		dev		The data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_io_destroy(dev_data_t* dev)
{
	int	i;

	dev_io_close(dev);	// Just in case.

	for (i = 0; i < DEV_IO_STREAM_QTY; i++)
	{
		if (dev->io.io_streams[i])
		{
			gsc_io_destroy(dev, dev->io.io_streams[i]);
		}
	}
}



/******************************************************************************
*
*	Function:	dev_io_open
*
*	Purpose:
*
*		Perform I/O based initialization as the device is being opened.
*
*	Arguments:
*
*		dev		The data for the device of interest.
*
*	Returned:
*
*		0		All went well.
*		< 0		The code for the error seen.
*
******************************************************************************/

int dev_io_open(dev_data_t* dev)
{
	int	i;

	dev_io_close(dev);	// Just in case.

	for (i = 0; i < DEV_IO_STREAM_QTY; i++)
	{
		if (dev->io.io_streams[i])
		{
			if (dev->io.io_streams[i]->dev_io_open)
			{
				(dev->io.io_streams[i]->dev_io_open)(dev, dev->io.io_streams[i]);
			}
		}
	}

	return(0);
}



/******************************************************************************
*
*	Function:	dev_io_read_select
*
*	Purpose:
*
*		Select the dev_io_t structure for the referenced I/O read stream.
*
*	Arguments:
*
*		dev		The data for the device of interest.
*
*		count	The number of bytes to transfer, with the I/O selector encoded
*				in the 0xF0000000 nibble.
*
*	Returned:
*
*		0		All went well.
*		< 0		The code for the error seen.
*
******************************************************************************/

dev_io_t* dev_io_read_select(dev_data_t* dev, size_t select)
{
	dev_io_t*	io;

	// We have only one input stream.
	io	= &dev->io.rx;
	return(io);
}



/******************************************************************************
*
*	Function:	dev_io_write_select
*
*	Purpose:
*
*		Select the dev_io_t structure for the referenced I/O write stream.
*
*	Arguments:
*
*		dev		The data for the device of interest.
*
*		count	The number of bytes to transfer, with the I/O selector encoded
*				in the 0xF0000000 nibble.
*
*	Returned:
*
*		0		All went well.
*		< 0		The code for the error seen.
*
******************************************************************************/

dev_io_t* dev_io_write_select(dev_data_t* dev, size_t select)
{
	dev_io_t*	io;

	// We have only one output stream.
	io	= &dev->io.tx;
	return(io);
}



