// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/driver/read.c $
// $Rev: 51526 $
// $Date: 2022-10-04 17:24:06 -0500 (Tue, 04 Oct 2022) $

// HPDI32: Device Driver: source file

#include "main.h"



//*****************************************************************************
static long _rx_available(dev_data_t* dev, size_t bytes)
{
	u32	almost;
	u32	bsr;
	u32	fwr;
	u32	rar;

	for (;;)	/* A convenience loop. */
	{
		if (dev->cache.reg_rfwr)
		{
			// The Rx FIFO Words Register is available.

			if (dev->cache.reg_fwr_update)
			{
				// In some versions of firmware the
				// RFWR may return zero when full.
				fwr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_rfwr_32);

				if (fwr)
				{
					// The Rx FIFO has data.
					// Available = RFWR
					bytes	= fwr;
					break;
				}

				bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);

				if ((bsr & D15) == 0)
				{
					// The Rx FIFO is Full.
					// Available = Rx FIFO Size
					bytes	= dev->cache.fifo_size_rx;
					break;
				}

				// The Rx FIFO is Empty.
				// Available = 0
				bytes	= 0;
				break;
			}
		}

		bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);

		if (dev->cache.reg_rfsr)
		{
			// The Rx FIFO Size Register is available.

			if ((bsr & HPDI32_BSR_RX_NOT_FULL) == 0)
			{
				// The Rx FIFO is full.
				// Available = Rx FIFO Size
				bytes	= dev->cache.fifo_size_rx;
				break;
			}

			if ((bsr & HPDI32_BSR_RX_NOT_AF) == 0)
			{
				// The Rx FIFO is Almost Full.
				// Available >= Rx FIFO Size - Rx Almost Full
				rar		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_rar_32);
				almost	= HPDI32_RAR_DECODE_FULL(rar);
				bytes	= dev->cache.fifo_size_rx - almost;
				break;
			}

			if (bsr & HPDI32_BSR_RX_NOT_AE)
			{
				// The Rx FIFO is not Almost Empty.
				// Available > Rx Almost Empty
				rar		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_rar_32);
				almost	= HPDI32_RAR_DECODE_EMPTY(rar);
				bytes	= almost + 1;
				break;
			}

			if (bsr & HPDI32_BSR_RX_NOT_EMPTY)
			{
				// The Rx FIFO is not Empty.
				// Available >= 1
				bytes	= 1;
				break;
			}

			// The Rx FIFO is Empty.
			// Available = 0
			bytes	= 0;
			break;
		}

		// The RFWR and RFSR are not available.

		if (bsr & HPDI32_BSR_RX_NOT_AE)
		{
			// The Rx FIFO is not Almost Empty.
			// Available > Rx Almost Empty
			rar		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_rar_32);
			almost	= HPDI32_RAR_DECODE_EMPTY(rar);
			bytes	= almost + 1;
			break;
		}

		if (bsr & HPDI32_BSR_RX_NOT_EMPTY)
		{
			// The Rx FIFO is not Empty.
			// Available >= 1
			bytes	= 1;
			break;
		}

		// The Rx FIFO is Empty.
		// Available = 0
		bytes	= 0;
		break;
	}

	bytes	*= dev->io.rx.bytes_per_sample;
	return((long) bytes);
}



//*****************************************************************************
static void _dev_io_sw_init(dev_data_t* dev, dev_io_t* io)
{
	io->auto_start			= HPDI32_AUTO_START_DEFAULT;
	io->bmdma_threshold		= HPDI32_IO_BMDMA_THRESH_DEFAULT;
	io->bytes_per_sample	= HPDI32_IO_DATA_SIZE_BITS_DEFAULT / 8;
	io->data_size_bits		= HPDI32_IO_DATA_SIZE_BITS_DEFAULT;
	io->io_mode				= HPDI32_IO_MODE_DEFAULT;
	io->overrun_check		= HPDI32_IO_ERROR_DEFAULT;
	io->pio_threshold		= HPDI32_IO_PIO_THRESHOLD_DEFAULT;
	io->timeout_s			= HPDI32_IO_TIMEOUT_DEFAULT;
	io->underrun_check		= HPDI32_IO_ERROR_DEFAULT;
}



//*****************************************************************************
static void _dev_io_close(dev_data_t* dev, dev_io_t* io)
{
	io->dma_channel	= NULL;
	os_mem_dma_close(&io->mem);
}



//*****************************************************************************
static void _dev_io_open(dev_data_t* dev, dev_io_t* io)
{
	_dev_io_sw_init(dev, io);
	os_mem_dma_open(&io->mem);
}



//*****************************************************************************
static int _dev_io_startup(dev_data_t* dev, dev_io_t* io)
{
	int			apply	= 0;
	u32			bcr;
	u32			bsr;
	long		ret		= 0;

	// Check for Rx FIFO overruns and underruns.

	if ((dev->cache.over_under_run) &&
		(io->overrun_check || io->underrun_check))
	{
		bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);

		if ((io->underrun_check) && (bsr & D22))
			ret	= -EIO;

		if ((io->overrun_check) && (bsr & D23))
			ret	= -EIO;
	}

	// Configure the DMDMA direction bit.
	bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (((bcr & HPDI32_BCR_DMDMA_DIR_TX) == 0)	&&
		(io->io_mode == GSC_IO_MODE_DMDMA)	&&
		(dev->cache.dmdma_ch_1 == 0))
	{
		// We've only got one DMDMA channel.
		// Clear the direction bit for RX operations.
		apply	= 1;
		bcr		&= ~HPDI32_BCR_DMDMA_DIR_TX;
	}

	if (((bcr & HPDI32_BCR_SING_CYC_DIS) == 0)	&&
		(io->io_mode == GSC_IO_MODE_DMDMA)	&&
		(dev->cache.single_cycle_disable))
	{
		apply	= 1;
		bcr		|= HPDI32_BCR_SING_CYC_DIS;
	}

	if (apply)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bcr_32, bcr);

	// Apply the auto-start setting.

	if ((ret == 0) && (io->auto_start))
		io_auto_start(dev, io);

	return(ret);
}



//*****************************************************************************
static long _dev_pio_available(dev_data_t* dev, dev_io_t* io, size_t bytes)
{
	bytes	= _rx_available(dev, bytes);
	return((long) bytes);
}



//*****************************************************************************
static long _dev_pio_xfer(
	dev_data_t*		dev,
	dev_io_t*		io,
	const os_mem_t*	mem,
	size_t			bytes,
	os_time_tick_t	st_end)
{
	long	qty;

	qty	= gsc_read_pio_work(dev, io, mem, bytes, st_end);
	return(qty);
}



//*****************************************************************************
static long _dev_bmdma_available(dev_data_t* dev, dev_io_t* io, size_t bytes)
{
	s32	available;

	available	= _rx_available(dev, bytes);

	if (available >= (s32) bytes)
		bytes	= available;
	else if (available < io->bmdma_threshold)
		bytes	= 0;
	else
		bytes	= available;

	return((long) bytes);
}



//*****************************************************************************
static long _dev_bmdma_xfer(
	dev_data_t*		dev,
	dev_io_t*		io,
	const os_mem_t*	mem,
	size_t			bytes,
	os_time_tick_t	st_end)
{
	long			qty;
	long			samples	= (long) bytes / io->bytes_per_sample;
	gsc_dma_setup_t	setup;

	if (samples <= io->pio_threshold)
	{
		qty	= gsc_read_pio_work(dev, io, mem, bytes, st_end);
	}
	else
	{
		memset(&setup, 0, sizeof(gsc_dma_setup_t));
		setup.alt		= dev;
		setup.dev		= dev;
		setup.io		= io;
		setup.mem		= mem;
		setup.st_end	= st_end;
		setup.bytes		= bytes;
		setup.ability	= GSC_DMA_CAP_BMDMA_READ;

		setup.mode		= GSC_DMA_MODE_BLOCK_DMA
						| GSC_DMA_MODE_INPUT_ENABLE
						| GSC_DMA_MODE_BURSTING_LOCAL
						| GSC_DMA_MODE_INTERRUPT_WHEN_DONE
						| GSC_DMA_MODE_LOCAL_ADRESS_CONSTANT
						| GSC_DMA_MODE_PCI_INTERRUPT_ENABLE;

		if (io->bytes_per_sample == 4)
			setup.mode	|= GSC_DMA_MODE_SIZE_32_BITS;
		else if (io->bytes_per_sample == 2)
			setup.mode	|= GSC_DMA_MODE_SIZE_16_BITS;
		else
			setup.mode	|= GSC_DMA_MODE_SIZE_8_BITS;

		setup.dpr		= GSC_DMA_DPR_BOARD_TO_HOST
						| GSC_DMA_DPR_END_OF_CHAIN
						| GSC_DMA_DPR_TERMINAL_COUNT_IRQ;

		qty	= gsc_dma_perform(&setup);
	}

	return(qty);
}



//*****************************************************************************
static long _dev_dmdma_available(dev_data_t* dev, dev_io_t* io, size_t bytes)
{
	// Procede as if the input stream is continuous. If the input stream is not
	// continuous, then the application must alter its means of reading data.
	// The application may read the leading portion via DMDMA, but the trailing
	// Almost Empty sized portion must be read by DMA or PIO.
	return((long) bytes);
}



//*****************************************************************************
static long _dev_dmdma_xfer(
	dev_data_t*		dev,
	dev_io_t*		io,
	const os_mem_t*	mem,
	size_t			bytes,
	os_time_tick_t	st_end)
{
	long			qty;
	long			samples	= (long) bytes / io->bytes_per_sample;
	gsc_dma_setup_t	setup;

	if (samples <= io->pio_threshold)
	{
		qty	= gsc_read_pio_work(dev, io, mem, bytes, st_end);
	}
	else
	{
		memset(&setup, 0, sizeof(gsc_dma_setup_t));
		setup.alt		= dev;
		setup.dev		= dev;
		setup.io		= io;
		setup.mem		= mem;
		setup.st_end	= st_end;
		setup.bytes		= bytes;
		setup.ability	= GSC_DMA_CAP_DMDMA_READ;

		setup.mode		= GSC_DMA_MODE_DM_DMA
						| GSC_DMA_MODE_INPUT_ENABLE
						| GSC_DMA_MODE_BURSTING_LOCAL
						| GSC_DMA_MODE_INTERRUPT_WHEN_DONE
						| GSC_DMA_MODE_LOCAL_ADRESS_CONSTANT
						| GSC_DMA_MODE_PCI_INTERRUPT_ENABLE;

		if (io->bytes_per_sample == 4)
			setup.mode	|= GSC_DMA_MODE_SIZE_32_BITS;
		else if (io->bytes_per_sample == 2)
			setup.mode	|= GSC_DMA_MODE_SIZE_16_BITS;
		else
			setup.mode	|= GSC_DMA_MODE_SIZE_8_BITS;

		setup.dpr		= GSC_DMA_DPR_BOARD_TO_HOST
						| GSC_DMA_DPR_END_OF_CHAIN
						| GSC_DMA_DPR_TERMINAL_COUNT_IRQ;

		qty	= gsc_dma_perform(&setup);
	}

	return(qty);
}



/******************************************************************************
*
*	Function:	dev_read_create
*
*	Purpose:
*
*		Perform a one-tine setup of the Read Input streaming I/O structure.
*
*	Arguments:
*
*		dev		The data for the device of interest.
*
*		io		The I/O structure for this stream.
*
*	Returned:
*
*		None.
*
******************************************************************************/

int dev_read_create(dev_data_t* dev, dev_io_t* io)
{
	int	ret;

	io->io_reg_offset		= GSC_REG_OFFSET(HPDI32_GSC_FDR);
	io->io_reg_vaddr		= dev->vaddr.gsc_fdr_32;
	io->is_tx				= 0;

	io->dev_io_sw_init		= _dev_io_sw_init;
	io->dev_io_close		= _dev_io_close;
	io->dev_io_open			= _dev_io_open;
	io->dev_io_startup		= _dev_io_startup;
	io->dev_pio_available	= _dev_pio_available;
	io->dev_bmdma_available	= _dev_bmdma_available;
	io->dev_dmdma_available	= _dev_dmdma_available;
	io->dev_pio_xfer		= _dev_pio_xfer;
	io->dev_bmdma_xfer		= _dev_bmdma_xfer;
	io->dev_dmdma_xfer		= _dev_dmdma_xfer;

	io->wait.abort			= HPDI32_WAIT_IO_RX_ABORT;
	io->wait.done			= HPDI32_WAIT_IO_RX_DONE;
	io->wait.error			= HPDI32_WAIT_IO_RX_ERROR;
	io->wait.timeout		= HPDI32_WAIT_IO_RX_TIMEOUT;

	ret	= gsc_io_create(dev, io, _8M);
	return(ret);
}


