// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/xfer/read.c $
// $Rev: 46888 $
// $Date: 2020-02-24 15:10:26 -0600 (Mon, 24 Feb 2020) $

// HPDI32: Sample Application: source file

#include "main.h"



// prototypes *****************************************************************

static int _callback(int fd, line_data_t* line_data);



// variables ******************************************************************

static const arg_type_s32_range_t	_arg_1_set[]	=
{
	// min	max		desc
	{ 1,	_32M,	"Read blocks containing this many samples."	},
};

static const arg_entry_t		_arg_1	=
{
	/* type	*/	ARG_TYPE_S32_RANGE,
	/* qty	*/	SIZEOF_ARRAY(_arg_1_set),
	/* list	*/	(void*) _arg_1_set
};

static const arg_type_s32_range_t	_arg_2_set[]	=
{
	// min	max		desc
	{ 1,	_1M,	"Read this number of blocks."	},
};

static const arg_entry_t		_arg_2	=
{
	/* type	*/	ARG_TYPE_S32_RANGE,
	/* qty	*/	SIZEOF_ARRAY(_arg_2_set),
	/* list	*/	(void*) _arg_2_set
};

static const arg_entry_t*		_arg_list[]	=
{
	&_arg_1,
	&_arg_2
};

const command_entry_t			cmd_read	=
{
	/* cmd		*/	"read",
	/* desc		*/	"Read data from the device.",
	/* function	*/	_callback,
	/* arg_qty	*/	SIZEOF_ARRAY(_arg_list),
	/* arg_list	*/	_arg_list
};



//*****************************************************************************
static void _show_data_set(s32 samples, s32 blocks)
{
	long long	qty;

	gsc_label("Data Set");
	qty	= (long long) samples * blocks;
	gsc_label_long_comma(qty);
	printf(" samples, ");
	qty	*=  global.rx_io_data_size / 8;
	gsc_label_long_comma(qty);
	printf(" bytes\n");
}



//*****************************************************************************
static void* _buffer_alloc(s32 samples, long* bytes)
{
	void*	ptr;

	gsc_label("Allocate Buffer");
	bytes[0]	= (long) samples * (global.rx_io_data_size / 8);
	ptr			= malloc(bytes[0]);
	printf("%s  (", ptr ? "PASS" : "FAIL");
	gsc_label_long_comma(bytes[0]);
	printf(" bytes)\n");
	return(ptr);
}



//*****************************************************************************
static void* _buffer_free(void* ptr)
{
	gsc_label("Free Buffer");
	free(ptr);
	printf("Done\n");
	return(NULL);
}



//*****************************************************************************
static int _read_data(int fd, long bytes, s32 blocks, void* ptr)
{
	int				errs	= 0;
	long			l;
	time_t			now;
	double			rate;
	int				ret;
	double			secs;
	time_t			start;
	os_time_us_t	t1;
	os_time_us_t	t2;
	long long		total	= 0;

	gsc_label("Reading");
	os_time_get_us(&t1);
	start	= time(NULL);

	for (l = 0; l < blocks; l++)
	{
		ret	= hpdi32_read(fd, ptr, bytes);

		if (ret < 0)
		{
			os_time_get_us(&t2);
			errs++;
			printf("ERROR: hpdi32_read(): error = %d\n", ret);
			break;
		}

		if (ret == 0)
		{
			now	= time(NULL);

			if (now >= (start + 2))
			{
				os_time_get_us(&t2);
				errs++;
				printf("ERROR: hpdi32_read() returning no data\n");
				break;
			}

			os_sleep_ms(1);
			continue;
		}

		total	+= ret;
		start	= time(NULL);
	}

	if (errs == 0)
	{
		os_time_get_us(&t2);
		printf("Done\n");

		gsc_label("Performance");
		secs	= (float) (t2.tv_sec - t1.tv_sec)
				+ (float) (0.000001 * t2.tv_usec)
				- (float) (0.000001 * t1.tv_usec);
		rate	= (long double) total / (global.rx_io_data_size / 8) / secs;
		gsc_label_long_comma(total / (global.rx_io_data_size / 8));
		printf(	" samples, ");
		gsc_label_float_comma((double) secs, 5, 3);
		printf(	" seconds, rate ");
		gsc_label_float_comma((double) rate, 5, 3);
		printf(	" S/S\n");
	}

	return(errs);
}



//*****************************************************************************
static int _callback(int fd, line_data_t* line_data)
{
	s32		blocks	= 0;
	long	bytes;
	int		errs	= 0;
	void*	ptr;
	s32		samples	= 0;

	if (line_data)
	{
		errs	= config_arg_qty_verify(line_data, 3);	// cmd = qty, times

		if (errs == 0)
			errs	= config_arg_validate_s32_range(line_data, 1, &cmd_read, &samples);

		if (errs == 0)
			errs	= config_arg_validate_s32_range(line_data, 2, &cmd_read, &blocks);

		if (errs == 0)
			config_display_verbose(fd, line_data, &cmd_read, 0);

		if (errs == 0)
		{
			gsc_label_level_inc();

			_show_data_set(samples, blocks);
			ptr	= _buffer_alloc(samples, &bytes);

			if (ptr == NULL)
			{
				errs++;
			}
			else
			{
				errs	+= _read_data(fd, bytes, blocks, ptr);
				_buffer_free(ptr);
			}

			gsc_label_level_dec();
		}
	}
	else
	{
		// Print the current value.
		// There isn't anything to print.
	}

	return(errs);
}


