// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/utils/util_id.c $
// $Rev: 53382 $
// $Date: 2023-07-12 11:12:16 -0500 (Wed, 12 Jul 2023) $

// HPDI32: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	_id_device_pci
*
*	Purpose:
*
*		Identify the device using the PCI registers.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

static int _id_device_pci(int fd, int verbose)
{
	char	buf[128];
	u32		didr;
	int		errs	= 0;
	u32		reg;

	// Vendor ID ====================================================
	errs	+= hpdi32_reg_read(fd, -1, 0, GSC_PCI_9080_VIDR, &reg);
	sprintf(buf, "0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		strcat(buf, "(PLX)\n");
	}
	else
	{
		errs	= 1;
		strcat(buf, "(UNKNOWN) FAIL <---\n");
	}

	if (verbose)
	{
		gsc_label("Vendor ID");
		printf("%s", buf);
	}

	// Device ID ====================================================
	errs	+= hpdi32_reg_read(fd, -1, 0, GSC_PCI_9080_DIDR, &didr);
	sprintf(buf, "0x%04lX      ", (long) didr);

	if (didr == 0x9080)
	{
		strcat(buf, "(PCI9080)\n");
	}
	else if (didr == 0x9656)
	{
		strcat(buf, "(PCI9656)\n");
	}
	else
	{
		errs	= 1;
		strcat(buf, "(UNKNOWN) FAIL <---\n");
	}

	if (verbose)
	{
		gsc_label("Device ID");
		printf("%s", buf);
	}

	// Subsystem Vendor ID ==========================================
	errs	+= hpdi32_reg_read(fd, -1, 0, GSC_PCI_9080_SVID, &reg);
	sprintf(buf, "0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		strcat(buf, "(PLX)\n");
	}
	else
	{
		errs	= 1;
		strcat(buf, "(UNKNOWN) FAIL <---\n");
	}

	if (verbose)
	{
		gsc_label("Sub Vendor ID");
		printf("%s", buf);
	}

	// Subsystem ID =================================================
	errs	+= hpdi32_reg_read(fd, -1, 0, GSC_PCI_9080_SID, &reg);
	sprintf(buf, "0x%04lX      ", (long) reg);

	if (didr == 0x9080)
	{
		if (reg == 0x2400)
		{
			strcat(buf, "(HPDI32, 32-bit)\n");
		}
		else if (reg == 0x9080)
		{
			// A very old device.
			strcat(buf, "(HPDI32, 32-bit)\n");
		}
		else
		{
			errs	= 1;
			strcat(buf, "(UNKNOWN) FAIL <---\n");
		}
	}
	else if (didr == 0x9656)
	{
		if (reg == 0x2705)
		{
			strcat(buf, "(HPDI32, 64-bit)\n");
		}
		else
		{
			errs	= 1;
			strcat(buf, "(UNKNOWN) FAIL <---\n");
		}
	}
	else
	{
		errs	= 1;
		strcat(buf, "(UNKNOWN) FAIL <---\n");
	}

	if (verbose)
	{
		gsc_label("Subsystem ID");
		printf("%s", buf);
	}

	return(errs);
}



//*****************************************************************************
static int _get_prefix(int fd, char* dst)
{
	s32			bus;
	int			errs	= 0;
	s32			ff;
	const char*	post	= "";
	const char*	pre		= "";

	dst		+= strlen(dst);
	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_BUS_WIDTH, &bus);
	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_FORM_FACTOR, &ff);

	if (errs)
		sprintf(dst, " FAIL <---  (query error(s))");

	if (errs == 0)
	{
		switch (ff)
		{
			default:

				errs	= 1;
				sprintf(dst, " FAIL <---  (invalid form factor: %ld)", (long) ff);
				break;

			case HPDI32_QUERY_FF_UNKNOWN:						break;
			case HPDI32_QUERY_FF_PCI:		pre	= "PCI";		break;
			case HPDI32_QUERY_FF_PMC:		pre	= "PMC";		break;
			case HPDI32_QUERY_FF_CPCI:		pre	= "CPCI";		break;
			case HPDI32_QUERY_FF_PC104P:	pre	= "PC104P-";	break;
		}
	}

	if (errs == 0)
	{
		switch (bus)
		{
			default:

				errs	= 1;
				sprintf(dst, " FAIL <---  (invalid bus size: %ld)", (long) bus);
				break;

			case 32:	post	= "";	break;
			case 64:	post	= "64";	break;
		}
	}

	if (errs == 0)
		sprintf(dst, "%s%s-", pre, post);

	return(errs);
}



//*****************************************************************************
static int _get_transceivers(int fd, char* dst)
{
	int			errs	= 0;
	const char*	psz		= "";
	s32			xcvr;

	dst		+= strlen(dst);
	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_XCVR_TYPE, &xcvr);

	if (errs)
	{
		sprintf(dst, " FAIL <---  (query error(s))");
	}
	else
	{
		switch (xcvr)
		{
			default:

				errs	= 1;
				sprintf(dst, " FAIL <---  (invalid form factor: %ld)", (long) xcvr);
				break;

			case HPDI32_QUERY_XCVR_UNKNOWN:						break;
			case HPDI32_QUERY_XCVR_PECL:		psz	= "-PECL";	break;
			case HPDI32_QUERY_XCVR_RS485:		psz	= "-RS485";	break;
		}
	}

	if (errs == 0)
		sprintf(dst, "%s", psz);

	return(errs);
}



//*****************************************************************************
static int _get_memory_size(int fd, char* dst)
{
	int			errs		= 0;
	const char*	factor		= "";
	long		fifo		= 0;
	s32			rx			= 0;
	s32			tx			= 0;

	dst		+= strlen(dst);
	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_FIFO_SIZE_RX, &rx);
	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_FIFO_SIZE_TX, &tx);

	if (errs)
		sprintf(dst, " FAIL <---  (query error(s))");

	if (rx > 0)
		fifo	+= rx * 4;

	if (tx > 0)
		fifo	+= tx * 4;

	if ((fifo) && ((fifo % 1024) == 0))
	{
		fifo	/= 1024;
		factor	= "K";
	}

	if ((fifo) && ((fifo % 1024) == 0))
	{
		fifo	/= 1024;
		factor	= "M";
	}

	if ((errs == 0) && (fifo > 0))
		sprintf(dst, "-%ld%s", fifo, factor);

	return(errs);
}



//*****************************************************************************
static int _get_model(int fd, int index, char* dst)
{
	int		errs		= 0;
	s32		model;
	s32		type;

	dst		+= strlen(dst);
	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_DEVICE_TYPE, &type);
	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_MODEL, &model);

	if (errs)
	{
		sprintf(dst, " FAIL <---  (query error(s))");
	}
	else if (type != GSC_DEV_TYPE_HPDI32)
	{
		errs	= 1;
		sprintf(dst, " FAIL <---  (invalid device type: %ld)", (long) type);
	}
	else
	{
		switch (model)
		{
			default:
			case HPDI32_QUERY_MODEL_HPDI32:

				strcat(dst, "HPDI32");
				break;

			case HPDI32_QUERY_MODEL_HPDI32A:

				strcat(dst, "HPDI32A");
				break;

			case HPDI32_QUERY_MODEL_HPDI32AL:

				strcat(dst, "HPDI32AL");
				break;

			case HPDI32_QUERY_MODEL_HPDI32ALT:

				strcat(dst, "HPDI32ALT");
				break;

			case HPDI32_QUERY_MODEL_HPDI32B:

				strcat(dst, "HPDI32B");
				break;
		}
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	hpdi32_id_device
*
*	Purpose:
*
*		Identify the device.
*
*	Arguments:
*
*		fd		The handle to use to access the device.
*
*		index	The index of the device or -1 if we don't care.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int hpdi32_id_device(int fd, int index, int verbose)
{
	int		errs;
	char	model[0124]		= "";

	if (verbose)
		gsc_label_index("Device", index);

	errs	= _get_prefix(fd, model);

	if (errs == 0)
		errs	= _get_model(fd, index, model);

	if (errs == 0)
		errs	= _get_memory_size(fd, model);

	if (errs == 0)
		errs	= _get_transceivers(fd, model);

	if (verbose)
	{
		printf("%s\n", model);
		gsc_label_level_inc();
	}

	errs	+= _id_device_pci(fd, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}



